#!/usr/bin/env python3
"""
Professional Flask web application for Anon.
Provides a clean, modern interface for text anonymization.
"""
import os
import sys
import tempfile
import re
from datetime import datetime
from flask import Flask, render_template, request, jsonify, send_file, session, abort
from werkzeug.utils import secure_filename

# Add parent directory to path for imports
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from anonymizer_core import redact

# Initialize Flask app
app = Flask(__name__)
app.config['MAX_CONTENT_LENGTH'] = 16 * 1024 * 1024  # 16MB max file size
app.config['SECRET_KEY'] = os.environ.get('SECRET_KEY', 'anon-dev-key-change-in-production')

# Allowed file extensions
ALLOWED_EXTENSIONS = {'txt'}

def allowed_file(filename):
    """Check if file extension is allowed."""
    return '.' in filename and filename.rsplit('.', 1)[1].lower() in ALLOWED_EXTENSIONS

def apply_user_always_redact(text, words):
    """Apply user's always redact words to the text."""
    if not words:
        return text

    modified_text = text
    for word in words:
        if word.strip():
            # Create case-insensitive pattern
            pattern = re.compile(re.escape(word), re.IGNORECASE)
            # Find all matches and replace
            for match in pattern.finditer(modified_text):
                modified_text = modified_text.replace(match.group(0), "<REDACTED>")

    return modified_text

@app.route('/')
def index():
    """Main application page."""
    return render_template('index.html')

@app.route('/api/v1/anonymize', methods=['POST'])
def anonymize_text():
    """
    API endpoint for text anonymization.
    """
    try:
        if not request.content_type or not request.content_type.startswith('application/json'):
            return jsonify({'error': 'Content-Type must be application/json'}), 415
        try:
            data = request.get_json(force=False, silent=False)
        except Exception:
            return jsonify({'error': 'Malformed JSON'}), 400
        if not data:
            return jsonify({'error': 'No JSON data provided'}), 400
        text = data.get('text', '').strip()
        if not text:
            return jsonify({'error': 'No text provided'}), 400
        always_redact_words = data.get('always_redact_words', [])
        options = data.get('options', {})
        include_entity_count = options.get('include_entity_count', True)
        include_mapping = options.get('include_mapping', False)
        result = redact(text)
        result_text = result.text
        if always_redact_words:
            result_text = apply_user_always_redact(result_text, always_redact_words)
        response = {
            'success': True,
            'anonymized_text': result_text,
            'timestamp': datetime.now().isoformat(),
            'version': '1.0.0'
        }
        if include_entity_count:
            response['entity_count'] = len(result.mapping)
        if include_mapping:
            response['entities_found'] = list(result.mapping.keys())
        return jsonify(response)
    except Exception as e:
        app.logger.error(f"Anonymization error: {str(e)}")
        return jsonify({'error': 'Anonymization failed', 'details': str(e)}), 500

@app.route('/api/v1/upload', methods=['POST'])
def upload_file():
    """
    Handle file uploads for anonymization.

    Returns the anonymized content as a downloadable file.
    """
    try:
        if 'file' not in request.files:
            return jsonify({'error': 'No file provided'}), 400

        file = request.files['file']
        if file.filename == '':
            return jsonify({'error': 'No file selected'}), 400

        if not allowed_file(file.filename):
            return jsonify({'error': 'Only .txt files are allowed'}), 400

        # Read file content
        content = file.read().decode('utf-8')

        # Get always redact words from form data
        always_redact_words = request.form.get('always_redact_words', '').split(',')
        always_redact_words = [word.strip() for word in always_redact_words if word.strip()]

        # Perform anonymization
        result = redact(content)
        result_text = result.text

        # Apply user's always redact words if any
        if always_redact_words:
            result_text = apply_user_always_redact(result_text, always_redact_words)

        # Create temporary file for download
        with tempfile.NamedTemporaryFile(mode='w', suffix='.txt', delete=False) as tmp:
            tmp.write(result_text)
            tmp_path = tmp.name

        # Generate safe filename
        original_name = secure_filename(file.filename or "")
        base_name = os.path.splitext(original_name)[0]
        download_name = f"anonymized_{base_name}.txt"

        return send_file(
            tmp_path,
            as_attachment=True,
            download_name=download_name,
            mimetype='text/plain'
        )

    except Exception as e:
        app.logger.error(f"File upload error: {str(e)}")
        return jsonify({'error': 'File processing failed', 'details': str(e)}), 500

@app.route('/api/v1/health')
def health_check():
    """Health check endpoint."""
    return jsonify({
        'status': 'healthy',
        'timestamp': datetime.now().isoformat(),
        'version': '1.0.0',
        'service': 'Anon Web API'
    })

@app.route('/api/v1/info')
def api_info():
    """API information endpoint."""
    return jsonify({
        'name': 'Anon Web API',
        'version': '1.0.0',
        'description': 'Privacy-first text anonymization service',
        'endpoints': {
            'anonymize': '/api/v1/anonymize',
            'upload': '/api/v1/upload',
            'health': '/api/v1/health'
        },
        'features': [
            'Dual-layer anonymization (Presidio + text-anonymizer)',
            'File upload support',
            'Always redact words (user-managed)',
            'RESTful API',
            'Entity counting',
            'Secure file handling'
        ]
    })

@app.errorhandler(404)
def not_found(error):
    """Handle 404 errors."""
    return jsonify({'error': 'Endpoint not found'}), 404

@app.errorhandler(413)
def too_large(error):
    """Handle file too large errors."""
    return jsonify({'error': 'File too large (max 16MB)'}), 413

@app.errorhandler(500)
def internal_error(error):
    """Handle internal server errors."""
    return jsonify({'error': 'Internal server error'}), 500

if __name__ == '__main__':
    # Development server
    app.run(host='0.0.0.0', port=8080, debug=False)